library bizdoc;

import 'package:bizdoc/gen_l10n/app_localizations.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import 'authenticate/base.dart';
import 'core/app.dart';
import 'core/configuration.dart';
import 'models/profile.dart';
import 'pages/browse.dart';
import 'pages/compose.dart';
import 'pages/explore.dart';
import 'pages/home.dart';
import 'pages/notifications.dart';
import 'pages/server_url.dart';
import 'pages/settings.dart';
import 'pages/splash.dart';
import 'services/session.dart';

class BizDocApp extends StatelessWidget with WidgetsBindingObserver {
  AppState? _state;

  BizDocApp(
      {Key? key,
      String? serverAddress,
      Map<String, Widget Function(BuildContext context)>? resolve,
      required AuthenticationBase authentication,
      GoogleMaps? googleMaps})
      : super(key: key) {
    WidgetsFlutterBinding.ensureInitialized();
    WidgetsBinding.instance?.addObserver(this);
    _state = AppState(resolve, authentication,
        serverAddress: serverAddress, googleMaps: googleMaps);
  }
  @override
  Widget build(BuildContext context) {
    return Provider(
      create: (ctx) => _state,
      child: MaterialApp(
        debugShowCheckedModeBanner: false,
        onGenerateTitle: (context) => AppLocalizations.of(context)!.appTitle,
        localizationsDelegates: AppLocalizations.localizationsDelegates,
        supportedLocales: AppLocalizations.supportedLocales,
        theme: ThemeData(
          colorScheme: const ColorScheme.light(secondary: Colors.black),
          primarySwatch: Colors.blue,
        ),
        routes: {
          '/': (context) => const SplashPage(),
          NotificationPage.routeName: (context) => NotificationPage(),
          ExplorePage.routeName: (context) => ExplorePage(),
          HomePage.routeName: (context) => HomePage(),
          BrowsePage.routeName: (context) =>
              BrowsePage(folderId: FolderInfo.inbox),
          ComposePage.routeName: (context) => ComposePage(),
          //       SettingsPage.routeName: (context) => const SettingsPage(),
          ServerUrlPage.routeName: (context) => const ServerUrlPage(),
          //SignInPage.routeName: (context) => SignInPage()
        },
        initialRoute: '/',
        // home: Navigator(
        //   pages: [
        //     //MaterialPage(name: SplashPage.routeName, child: SplashPage())
        //   ],
        //   initialRoute: SplashPage.routeName,
        // ),
      ),
    );
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) async {
    try {
      switch (state) {
        case AppLifecycleState.resumed:
          session.online(true);
          break;
        case AppLifecycleState.paused:
          session.online(false);
          break;
        default:
      }
    } catch (e) {
      print(e);
    }
  }
}
