import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:syncfusion_flutter_charts/charts.dart';

import 'package:bizdoc/services/cube.dart' as c;
import 'package:bizdoc/services/session.dart';
import 'package:bizdoc/models/models.dart';
import 'package:bizdoc/models/profile.dart';
import 'package:bizdoc/pages/dashboard.dart';

/// Dashboard widget
class CubeAnalysisWidget extends StatefulWidget implements IDashboardWidget {
  const CubeAnalysisWidget({Key? key}) : super(key: key);

  @override
  _CubeAnalysisWidgetState createState() => _CubeAnalysisWidgetState();
}

class _CubeAnalysisWidgetState extends State<CubeAnalysisWidget> {
  @override
  Widget build(BuildContext context) {
    return Consumer<WidgetState>(builder: (context, value, child) {
      final cube = value.options.containsKey('cube')
          ? session.profile.cubes
              .firstWhere((element) => element.name == value.options['cube'])
          : session.profile.cubes.first;
      final xAxis = value.options.containsKey('xAxis')
          ? List<String>.from(value.options['xAxis'])
          : [cube.axes.first.name];
      final indices = value.options.containsKey('indices')
          ? List<String>.from(value.options['indices'])
          : null;
      final chartType = CubeView.mapChartType(value.options['chartType']);
      return FutureBuilder<List<CubeSeries>>(
        future: c.cube.series(cube.name, xAxis, indices: indices),
        builder: (context, snapshot) {
          if (snapshot.hasData) {
            final data = snapshot.data!.elementAt(0);
            switch (chartType) {
              case ChartType.pie:
                return SfCircularChart(
                  series: [
                    PieSeries(
                      dataSource: data.points,
                      xValueMapper: (datum, index) => datum.xLabel,
                      yValueMapper: (datum, index) => datum.y,
                    )
                  ],
                );
              case ChartType.doughnut:
              case ChartType.halfDoughnut:
                return SfCircularChart(
                  series: [
                    DoughnutSeries(
                      startAngle: chartType == ChartType.doughnut ? 360 : 270,
                      endAngle: chartType == ChartType.doughnut ? 0 : 90,
                      dataSource: data.points,
                      xValueMapper: (datum, index) => datum.xLabel,
                      yValueMapper: (datum, index) => datum.y,
                    )
                  ],
                );
              default:
                final series = <ChartSeries>[];
                switch (chartType) {
                  case ChartType.bar:
                    series.add(BarSeries<Point, String>(
                      dataSource: data.points,
                      xValueMapper: (datum, index) => datum.xLabel,
                      yValueMapper: (datum, index) => datum.y,
                    ));
                    break;
                  case ChartType.area:
                    series.add(AreaSeries<Point, String>(
                      dataSource: data.points,
                      xValueMapper: (datum, index) => datum.xLabel,
                      yValueMapper: (datum, index) => datum.y,
                    ));
                    break;
                  case ChartType.spline:
                    series.add(SplineSeries<Point, String>(
                      dataSource: data.points,
                      name: data.title,
                      xValueMapper: (datum, index) => datum.xLabel,
                      yValueMapper: (datum, index) => datum.y,
                    ));
                    break;
                  case ChartType.splineArea:
                    series.add(SplineAreaSeries<Point, String>(
                      dataSource: data.points,
                      name: data.title,
                      xValueMapper: (datum, index) => datum.xLabel,
                      yValueMapper: (datum, index) => datum.y,
                    ));
                    break;
                  case ChartType.stepLine:
                    series.add(StepLineSeries<Point, String>(
                      dataSource: data.points,
                      name: data.title,
                      xValueMapper: (datum, index) => datum.xLabel,
                      yValueMapper: (datum, index) => datum.y,
                    ));
                    break;
                  case ChartType.stackingBar:
                    series.add(StackedBarSeries<Point, String>(
                      dataSource: data.points,
                      name: data.title,
                      xValueMapper: (datum, index) => datum.xLabel,
                      yValueMapper: (datum, index) => datum.y,
                    ));
                    break;
                  case ChartType.stepArea:
                    series.add(StepAreaSeries<Point, String>(
                      dataSource: data.points,
                      name: data.title,
                      xValueMapper: (datum, index) => datum.xLabel,
                      yValueMapper: (datum, index) => datum.y,
                    ));
                    break;
                  case ChartType.stackingColumn:
                    series.add(StackedColumnSeries<Point, String>(
                      dataSource: data.points,
                      name: data.title,
                      xValueMapper: (datum, index) => datum.xLabel,
                      yValueMapper: (datum, index) => datum.y,
                    ));
                    break;
                  case ChartType.stackingLine:
                    series.add(StackedLineSeries<Point, String>(
                      dataSource: data.points,
                      name: data.title,
                      xValueMapper: (datum, index) => datum.xLabel,
                      yValueMapper: (datum, index) => datum.y,
                    ));
                    break;
                  case ChartType.stackingArea:
                    series.add(StackedAreaSeries<Point, String>(
                      dataSource: data.points,
                      name: data.title,
                      xValueMapper: (datum, index) => datum.xLabel,
                      yValueMapper: (datum, index) => datum.y,
                    ));
                    break;
                  default:
                    series.add(LineSeries<Point, String>(
                      dataSource: data.points,
                      xValueMapper: (datum, index) => datum.xLabel,
                      yValueMapper: (datum, index) => datum.y,
                    ));
                }
                return SfCartesianChart(
                  primaryXAxis: CategoryAxis(),
                  series: series,
                );
            }
          }
          return const Center(child: CircularProgressIndicator());
        },
      );
    });
  }
}
