import 'package:bizdoc/core/app.dart';
import 'package:bizdoc/models/profile.dart';
import 'package:bizdoc/services/session.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

class IDashboardWidget {}

/// state which can be consumed by a dashboard widget
class WidgetState {
  final WidgetInfo _widget;

  Map<String, dynamic> get options => _widget.options ?? {};
  String get name => _widget.name;
  WidgetState(this._widget);
}

/// show tiles from [Profile] widgets. Instantiate widget by name from [repository].
class DashboardPage extends StatefulWidget {
  static const routeName = 'dashboard';

  const DashboardPage({Key? key}) : super(key: key);

  @override
  _DashboardPageState createState() => _DashboardPageState();
}

class _DashboardPageState extends State<DashboardPage> {
  @override
  Widget build(BuildContext context) {
    return Consumer<AppState>(
        builder: (context, value, child) =>
            OrientationBuilder(builder: (context, orientation) {
              final children = <Widget>[];
              for (var element in session.profile.widgets) {
                final item = value.resolve(element.template, context);
                if (item != null) {
                  children.add(Provider(
                    create: (context) => WidgetState(element),
                    child: item,
                  ));
                }
              }

              return GridView.count(
                  crossAxisCount: orientation == Orientation.portrait ? 1 : 2,
                  children: children);
            }));
  }
}
